@@ -14,7 +14,7 @@ export const ResourceIcon = ({kind}: {kind: string}) => {
     const style: React.CSSProperties = {
         display: 'inline-block',
         verticalAlign: 'middle',
-        padding: '2px 4px',
+        padding: `${n <= 2 ? 2 : 0}px 4px`,
         width: '32px',
         height: '32px',
         borderRadius: '50%',
